/******************************************************************************
* Copyright (c) 2005,2006, Novell, Inc.  All rights reserved.
*
* Redistribution and use in source and binary forms, with or without 
* modification, are permitted provided that the following conditions 
* are met:
*
*	- Redistributions of source code must retain the above copyright notice,
*	this list of conditions and the following disclaimer.
*
*	- Redistributions in binary form must reproduce the above copyright notice,
*  	this list of conditions and the following disclaimer in the documentation
*	and/or other materials provided with the distribution.
*
*	- Neither the name of Novell nor the names of its contributors may be
*	used to endorse or promote products derived from this software without
*	specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
* POSSIBILITY OF SUCH DAMAGE.
******************************************************************************/

#ifndef SMASHCONFIG_HPP_GUARD
#define SMASHCONFIG_HPP_GUARD

#include <openwbem/OW_config.h>
#include <openwbem/OW_String.hpp>
#include <openwbem/OW_XMLNode.hpp>
#include <openwbem/OW_Array.hpp>
#include <openwbem/OW_Mutex.hpp>
#include <openwbem/OW_ConfigOpts.hpp>

namespace OMCSmashConfig
{

/**
 * The AggregateNamespace structure is used to represent namespaces
 * that will be used in the process of aggregating logical devices
 * and physical elements. It contains the information that is specified
 * in the following XML:
 * <aggregate_namespace name="ipmi" priority="1">
 *		smash/ipmi
 * </aggregate_namespace>
 */
struct AggregateNamespace
{
	/**
	 * Construct a new empty AggregateNamespace object.
	 */
	AggregateNamespace();

	/**
	 * Construct a new AggregateNamespace object from an existing
	 * AggregateNamespace object.
	 * @parm arg The AggregateNamespace object to copy the new object from.
	 */
	AggregateNamespace(
		const AggregateNamespace& arg);

	/**
	 * Assign the values from another AggregateNamespace object to this one.
	 * @parm arg The AggregateNamespace object to assign the values from.
	 * @return A reference to this AggregateNamespace object after the 
	 * 		assignment.
	 */
	AggregateNamespace& operator= (
		const AggregateNamespace& arg);

	OW_NAMESPACE::String m_name;
	OW_NAMESPACE::String m_namespace;
	OW_NAMESPACE::UInt32 m_priority;
};

typedef OW_NAMESPACE::Array<AggregateNamespace> AggregateNamespaceArray;

/**
 * The IgnoreInstance structure represents hardware that should be
 * completely ignored by the omc-smash implementation. It contains
 * information from the following XML:
 * <ignore_instance
 *      class_name="OMC_PowerSupply"
 *      match_property="DeviceID"
 *      match_value="10.0" />
 */
struct IgnoreInstance
{
	/**
	 * Construct a new IgnoreInstance object from an XMLNode
	 * @parm node An XMLNode that is expected to point to an 
	 *		<ignore_instance> XML tag.
	 */
	IgnoreInstance(
		const OW_NAMESPACE::XMLNode& node);

	/**
	 * Construct a new empty IgnoreInstance object.
	 */
	IgnoreInstance();

	/**
	 * Construct a new IgnoreInstance object from an existing
	 * IgnoreInstance object.
	 * @parm arg The IgnoreInstance object to copy the new object from.
	 */
	IgnoreInstance(
		const IgnoreInstance& arg);

	/**
	 * Assign the values from another IgnoreInstance object to this one.
	 * @parm arg The IgnoreInstance object to assign the values from.
	 * @return A reference to this IgnoreInstance object after the 
	 * assignment.
	 */
	IgnoreInstance& operator= (
		const IgnoreInstance& arg);

	OW_NAMESPACE::String m_className;
	OW_NAMESPACE::String m_matchProp;
	OW_NAMESPACE::String m_matchValue;
};
typedef OW_NAMESPACE::Array<IgnoreInstance> IgnoreInstanceArray;

/**
 * @return the directory name where the configuration files area stored.
 */
OW_NAMESPACE::String getConfigDir();

/**
 * @return true If the configuration is invalid. Otherwise false.
 */
bool isInvalid();

/**
 * Read all of the configuration data.
 */
void readConfig();

/**
 * @return the Smash namespace from the configuration.
 */
OW_NAMESPACE::String getSmashNs();

/**
 * @return the IPMI namespace from the configuration.
 */
OW_NAMESPACE::String getIpmiNs();

/**
 * @return the HAL namespace from the configuration.
 */
OW_NAMESPACE::String getHalNs();

/**
 * @return the Smash staging namespace from the configuration.
 */
OW_NAMESPACE::String getSmashStageNs();

/**
 * @return All the namespaces that are involved in the logical device/
 *		physical element aggregation.
 */
AggregateNamespaceArray getAggregateNamespaces();

/**
 * Determine the namespace that is associated with the given name
 * @parm name The name to get the namespace for (e.g. ipmi)
 * @return The namespace associated with the given name if found.
 *		Otherwise an empty string.
 */
OW_NAMESPACE::String getAggregateNamespace(const OW_NAMESPACE::String& name);

/**
 * Get all of the IgnoreInstance objects from the omc-smash
 * configuration with optional filtering on a given class name.
 * @param className (Optional) If provided, only IgnoreInstance objects
 * 		with the given class name will be returned.
 * @return An array of IgnoreInstance objects that match the search
 * 		criteria.
 */
IgnoreInstanceArray getIgnoreInstances(
	const OW_NAMESPACE::String& className=OW_NAMESPACE::String());

/**
 * Get the namespaces that need to be looked at when handling a given
 * class name.
 * @param omcResultClass The class name to get the aggregate namespaces for.
 * @param nsToCombine An output StringArray that will hold the namespaces
 *      to process for the given classname.
 */
void getAggregatedNamespaceList(
    const OW_NAMESPACE::String& omcResultClass,
    OW_NAMESPACE::StringArray& nsToCombine);

const char* const INTEROP_NS_opt = OW_NAMESPACE::ConfigOpts::INTEROP_SCHEMA_NAMESPACE_opt; 

}	// End of namespace OMCSmashConfig

#endif	// SMASHCONFIG_HPP_GUARD
